import reg
import signal

from fja_checker import check_task, dfa_transform, nfa_transform, check_alphabets, check_empty

class WebChecker():
    def __init__(self, student_string: str, task: str):
        self.student_string = student_string
        self.task = task
        self.eq = None
        self.ok = False

    def dfa_task(self, teacher_type, teacher_string, task, student_string):
        student_solution = dfa_transform(student_string, task)
        teacher_solution = dfa_transform(teacher_string, teacher_type)

        check_empty(student_solution=student_solution,
                    teacher_solution=teacher_solution)

        check_alphabets(student_alpha=student_solution.characters,
                        teacher_alpha=teacher_solution.characters, task=task)

        self.eq = reg.DFA.is_equivalent(student_solution, teacher_solution)
        if check_task(student_solution, task) and self.eq:
            self.ok = True

    def compare(self, teacher_string: str, teacher_type: str):
        signal.alarm(50)
        try:
            self.dfa_task(teacher_type=teacher_type, teacher_string=teacher_string,
                          task=self.task, student_string=self.student_string)

        except Exception as ex:
            print("Error inside of web checker:", ex.args)
            exit(1)

    def convert(self, student_type):
        signal.alarm(50)

        try:
            parser = reg.Parser()
            # todo nfas and so on

            if student_type in {"DFA", "TOT", "MIN", "TOC", "MIC"}:
                automaton = parser.str_to_dfa(self.student_string)
            elif student_type == "NFA":
                automaton = parser.str_to_nfa(self.student_string).determinize()
            elif student_type == "GRA":
                automaton = parser.str_to_reggrammar(self.student_string).reggrammar_to_nfa().determinize()
            elif student_type == "REG":
                automaton = parser.str_to_regex(self.student_string).regex_to_efa().eliminate_epsilon().determinize()

            if self.task == "DFA":
                return parser.dfa_to_str(automaton)
            if self.task == "TOT":
                return parser.dfa_to_str(automaton.total())
            if self.task == "MIN":
                return parser.dfa_to_str(automaton.minimize())
            if self.task == "MIC":
                return parser.dfa_to_str(automaton.minimize().canonize())

            print("Uncovered convert combination!")
            return

        except Exception as ex:
            print("Error inside of web checker:", ex.args)
            exit(1)